// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICORE_LOCKED_ALLOCATOR_H_INCLUDED
#define YAMICORE_LOCKED_ALLOCATOR_H_INCLUDED

#include "allocator.h"
#include "dll.h"

#include <mutex>

namespace yami
{

namespace core
{

/// \brief Locking (thread-safe) wrapper for the block-based allocator.
class DLL locked_allocator : public allocator
{
public:
    locked_allocator() {}
    
    void set_working_area(void * buf, std::size_t size)
    {
        alloc_.set_working_area(buf, size);
    }

    virtual void * allocate(std::size_t requested_size)
    {
        std::lock_guard<std::mutex> lock(mtx_);
        
        void * result = alloc_.allocate(requested_size);
        
        return result;
    }

    virtual void deallocate(const void * p)
    {
        std::lock_guard<std::mutex> lock(mtx_);
            
        alloc_.deallocate(p);
    }

    void get_free_size(std::size_t & biggest, std::size_t & all) const
    {
        std::lock_guard<std::mutex> lock(mtx_);
            
        alloc_.get_free_size(biggest, all);
    }

private:
    locked_allocator(const locked_allocator &);
    void operator=(const locked_allocator &);

    non_locked_allocator alloc_;
    mutable std::mutex mtx_;
};

} // namespace core

} // namespace yami

#endif // YAMICORE_LOCKED_ALLOCATOR_H_INCLUDED
